"""
Eporner API Wrapper
~~~~~~~~~~~~~~~~~~~

An API wrapper for the Eporner API.
:copyright: (c) 2021-present boobfuck
:license: MIT, see LICENSE for more details.

"""

__title__     = 'eporner'
__author__    = 'boobfuck'
__license__   = 'MIT'
__copyright__ = 'Copyright 2021-present boobfuck'
__version__   = '1.0.1'

from .http import *
from .enums import *
from .errors import *
from .client import *



"""
The MIT License (MIT)

Copyright (c) 2021-present boobfuck

Permission is hereby granted, free of charge, to any person obtaining a
copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE.
"""

from __future__ import annotations

import asyncio

from .http import *
from .enums import *

class Client:
    def __init__(self) -> None:
        self.http = HTTP()

    async def get_videos(
        self,
        query: str,
        *,
        per_page: int = 30,
        page: int = 1,
        thumbsize: Thumbsize = Thumbsize.medium,
        order: Order = Order.latest,
        gay_content: GayContent = GayContent.none,
        low_quality: LowQuality = LowQuality.include
    ) -> Result:
        """
        Gets the videos.

        ### Parameters
        query (str) - The search query. Special value "all" can be passed to query for all videos.
        per_page (int) - Limits the number of results per page. Valid range is 1 - 1000. Default: 30.
        page (int) - The results page number. Valid range is 1 - 1000000, but no more than ``total_pages`` received in response. Default: 1.
        thumbsize (Thumbsize) - The thumbail size. Default: ``Thumbsize.medium``.
        order (Order) - How results should be sorted. Default: ``Order.latest```.
        gay_content (GayContent) - Whether to include gay content or not. Default ``GayContent.none``.
        low_quality (LowQuality) - Whether to include low quality content or not. Default: ``LowQuality.include``.
        """

        params = {
            'query': query,
            'per_page': per_page,
            'page': page,
            'thumbsize': thumbsize.value,
            'order': order.value,
            'gay': gay_content.value,
            'lq': low_quality.value
        }

        data = await self.http.search_videos(params=params)
        return self.http.parse_result(data)

    async def get_id(self, id: str) -> Video:
        """
        Gets a video by it's unique ID.

        ### Parameters
        id (str) - The unique video ID.
        """

        params = {
            'id': id
        }
        data = await self.http.search_id(params=params)
        return self.http.parse_ids(data)

    # This current does not work.
    # I have contacted the API team for info.
    # Will update once I get a response, for
    # now, this method will stay.
    async def get_removed(self) -> list[str]:
        """
        Returns results for all removed IDs.
        """

        params = {
            'format': 'txt'
        }
        data = await self.http.search_removed(params=params)
        return self.http.parse_removed(data)

    def close(self) -> None:
        """Closes the internal client session."""
        asyncio.create_task(self.http.close())



"""
The MIT License (MIT)

Copyright (c) 2021-present boobfuck

Permission is hereby granted, free of charge, to any person obtaining a
copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE.
"""
import itertools

import aiohttp

from .errors import *
from .models import *

async def text_or_json(response):
    if response.headers['content-type'] == 'text/html':
        return await response.text(encoding='utf-8')
    else:
        return await response.json()

class Route:
    BASE = 'https://eporner.com/api/v2'

    def __init__(self, method, path):
        self.method = method
        self.path = path
        self.url = self.BASE + path

class HTTP:
    
    def __init__(self):
        self.user_agent = 'eporner.py (https://github.com/boobfuck/eporner.py)'
        headers = {
            'User-Agent': self.user_agent
        }
        self._session = aiohttp.ClientSession(headers=headers)

    def parse_removed(self, data):
        if data == []:
            raise NoResults('No results found')

        return data

    def parse_ids(self, data):
        if data == []:
            raise NoResults('No results found.')

        dthumb = data.get('default_thumb')
        thumbs_list = []
        thumbs = data.get('thumbs')
        for thumb in thumbs:
            thumbs_list.append(
                Thumbnail(
                    size=thumb.get('size'),
                    width=thumb.get('width'),
                    height=thumb.get('height'),
                    src=thumb.get('src')
                )
            )

        return Video(
            id=data.get('id'),
            title=data.get('title'),
            tags=data.get('keywords').split(', '),
            views=data.get('views'),
            rate=data.get('rate'),
            url=data.get('url'),
            added=data.get('added'),
            length_sec=data.get('length_sec'),
            length=data.get('length_min'),
            embed_url=data.get('embed'),
            default_thumb=Thumbnail(
                size=dthumb.get('size'),
                width=dthumb.get('width'),
                height=dthumb.get('height'),
                src=dthumb.get('src')
            ),
            thumbs=thumbs_list
        )

    def parse_videos(self, data):
        videos_list = []
        thumbs_list = []
        datas = data.get('videos')
        dthumbs = [data['default_thumb'] for data in datas]
        thumbs = [data['thumbs'] for data in datas]
        
        for thumb in itertools.chain.from_iterable(thumbs):
            thumbs_list.append(
                Thumbnail(
                    size=thumb.get('size'),
                    width=thumb.get('width'),
                    height=thumb.get('height'),
                    src=thumb.get('src')
                )
            )

        for data, dthumb in zip(datas, dthumbs):
            videos_list.append(
                Video(
                    id=data.get('id'),
                    title=data.get('title'),
                    tags=data.get('keywords').strip(', '),
                    views=data.get('views'),
                    rate=data.get('rate'),
                    url=data.get('url'),
                    added=data.get('added'),
                    length_sec=data.get('length_sec'),
                    length=data.get('length_min'),
                    embed_url=data.get('embed'),
                    default_thumb=Thumbnail(
                        size=dthumb.get('size'),
                        width=dthumb.get('width'),
                        height=dthumb.get('height'),
                        src=dthumb.get('src')
                        ),
                    thumbs=thumbs_list
                )
            )
        return videos_list

    def parse_result(self, data):
        if data.get('count') == 0:
            raise NoResults('No resuls found.')

        return Result(
            count=data.get('count'),
            current_page=data.get('page'),
            total_count=data.get('total_count'),
            total_pages=data.get('total_pages'),
            videos=self.parse_videos(data)
        )

    async def request(self, route, **kwargs):
        method = route.method
        url = route.url

        async with self._session.request(method, url, **kwargs) as response:
            data = await text_or_json(response)

            if response.status == 200:
                return data
            else:
                raise HTTPException(response, data)

    def search_videos(self, **kwargs):
        return self.request(Route('GET', '/video/search/'), **kwargs)

    def search_id(self, **kwargs):
        return self.request(Route('GET', '/video/id/'), **kwargs)

    def search_removed(self, **kwargs):
        return self.request(Route('GET', '/video/removed/'), **kwargs)

    async def close(self):
        await self._session.close()




"""
The MIT License (MIT)

Copyright (c) 2021-present boobfuck

Permission is hereby granted, free of charge, to any person obtaining a
copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE.
"""

import datetime
from dataclasses import dataclass

@dataclass
class Result:
    """
    The result of the query.
    ### Attributes
    count (int) - The number of videos returned on current result page.
    current_page (int) - The urrent result page number.
    total_count (int) -The total number of all videos found matching your criteria.
    total_pages (int) - The total number of pages with all results matching your criteria assuming current ``per_page`` value.
    videos (list) - A list of ``Video`` objects.
    """
    count: int
    current_page: int
    total_count: int
    total_pages: int
    videos: list

@dataclass
class Thumbnail:
    """
    The thumbnail information.

    ### Atrributes
    size (str) - The thumbnail size: big, medium, small.
    width (int) - The width of the thumbnail.
    height (int) - The height of the thumbnail.
    src (str) - The url of the thumbnail.
    """
    # TODO: Rename src to url?
    size: str
    width: int
    height: int
    src: str

@dataclass
class Video:
    """
    id (in) - Unique ID of the video.
    title (str) - The video title.
    tags (list) - The tags assigned to the video.
    views (int) - The Estimated number of video views.
    rate (float) - The video rate. Valid range is ``0.00`` to ``5.00``.
    url (str) - The URL of the video on Eporner.
    added (str) - The date of the video added on Eporner.
    length_sec (int) - The video length in seconds.
    length (str) - The video length in ``mm:ss`` format or ``hh:mm:ss`` format if video longer than 60 minutes.
    embed_url (str) - The embed url of the video.
    default_thumb (Thumbnail) - The video's default thumbnail information.
    thumbs (list) - A list of ``Thumbnail`` objects.
    """
    id: str
    title: str
    tags: list
    views: int
    rate: float
    url: str
    added: str
    length_sec: int
    length: str
    embed_url: str
    default_thumb: Thumbnail
    thumbs: list



import re
from setuptools import setup

with open('requirements.txt') as f:
    requirements = f.read().splitlines()

with open('eporner/__init__.py') as f:
    version = re.search(r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]', f.read(), re.MULTILINE)[1]

with open('README.md') as f:
    readme = f.read()

setup(
    name='eporner.py',
    author='boobfuck',
    url='https://github.com/boobfuck/eporner.py',
    project_urls={
        'Documentation': 'https://epornerpy.readthedocs.io/en/latest',
        'Issue tracker': 'https://github.com/boobfuck/eporner.py/issues'
    },
    version=version,
    packages=['eporner'],
    license='MIT',
    description='A simple asynchronous Eporner API wrapper.',
    long_description=readme,
    long_description_content_type='text/markdown',
    include_package_data=True,
    install_requires=requirements,
    python_requires='>=3.7.0'
)
